Polynomial

Struct Polynomial 

pub struct Polynomial { /* private fields */ }
A polynomial over the ML-DSA ring.

Dev note: The following structure does not necessarily need to be declared as public. There is no real scenario where this function needs to be called directly. However, in order to test the Debug and Display traits, it is necessary to use STD, so those canโ€™t be tested from inline tests in this file and the real unit tests are in a different crate. Thatโ€™s the reason why pub is used.

๐Ÿšจ Security ๐Ÿšจ

Polynomials themselves are not inherently secret since sometimes they are part of public keys and sometimes private keys. It is the responsibility of the caller to wrap sensitive instances in Secret<Polynomial>. Note: at the moment, nothing in this crate uses Secret<Polynomial>, so I have left the impl ZeroizablePrimitive commented-out.

Implementations

Source

impl Polynomial

pub const fn new() -> Self

Create a new polynomial with all coefficients set to zero.